function [cindex, err] = kmeans (data, n, k)
% [CINDEX, ERR] = KMEANS (DATA, N, K)
% Cluster DATA into K Clusters, 
% CINDEX(d)=r means data item d is in cluster r
% ERR is square of sum of distances of each pt from its centre
% Run algorithm N times, use value with smallest ERR
% if K is [kmin kmax] algorithm tries K from kmin to kmax
% but stops when reduction of ERR is small
% Simplified versions of author's clu_routines
% Dr Richard Mitchell, 15/08/03
if nargin < 3, k = 10; end
if nargin < 2, n = 3; end
[cindex, err] = donkmeans (data, n, k(1));
if length(k) > 1
   for ct = 2:length(k)
        [ncindex, nerr] = donkmeans(data, n, k(ct));
        if ct>2 & (err(ct-1)-nerr < 0.01*(err(ct-2)-err(ct-1)))
            break;                  % if marginal improvement to error, stop
        else
            err(ct) = nerr;         % remember the latest error
            cindex = ncindex;        % and the cluster index
        end
    end
end

function [index, err] = donkmeans (data, n, k)
err=inf;
for ct=1:n
   [kmndx, nerr] = dokmeans (data, k);
   if nerr<err, 
      index=kmndx;
      err=nerr;
   end
end

function [index, err] = dokmeans (data, k)
kmmax = 50;                                 % max 50 iterations
mind = min(data,[],1);		                % find limits of data - for initialise
maxd = max(data,[],1);
centres = rand(k, size(data,2));            % initialise centres randomly
for d=1:k                                   % within min,max
    centres(d,:) = mind + centres(d,:) .* (maxd - mind);
end
if k == 1
   index = ones(1, size(data,1));           % all points in cluster 1
else
   iterct = 1;                              % count of iterations of algorithm
   cnew = centres;                          % initialise mnew matrix and
   cdist = zeros(size(data,1),k);               % that for distances from kminfo.centres
   while iterct <= kmmax,                   % terminating condition kmax iterations
     for o=1:k                              % find distance of each point from centre
        cdist(:,o) = fdist(data, centres(o,:));
     end;
     [dummy index] = min(cdist');			% index(n) contains which cluster data(n) is in
     for o = 1:k                            % now find mean (centre) of each cluster
        datinc = data(index==o,:);          % get data in cluster o
        if size(datinc,1),				    % if cluster has many elements
           cnew(o,:)= mean(datinc,1);       % set its mean
        else								% if none in cluster
           cnew(o,:)= mean(data);           % force mean to data mean
        end;   
     end;   
     if centres==cnew,                      % if no change in cluster centres
        iterct = kmmax+1;                   % force algorithm to stop
     else 
        centres = cnew; 					% assign new cluster to centres
        iterct = iterct+1;                  % increment num iterations
     end;
  end;  
end;  

% now find distance^2 of each point from its cluster centre
err = 0;                                   % start from 0
for ct=1:k,									% add distance of each cluster from centre
   err = err + sum(fdist(data(index==ct,:),centres(ct,:)));
end;   
err = err / size(data, 1);                % now div by num data values
index = index';                             % so is a column vector

function dists = fdist(datset, dat)
% returns the square of distances between all items in datset and dat
dists = zeros(size(datset,1),1);
for ct = 1:size(datset,2)								% process each column in turn
   dists = dists + (datset(:,ct) - dat(ct)).^2;
end;

